@@ -1,4 +1,5 @@
 #! /bin/sh
+# $Cambridge: exim/exim-src/scripts/Configure-os.h,v 1.1 2004/10/06 15:07:40 ph10 Exp $
 
 # Shell script to create a link to the appropriate OS-specific header file.
 
@@ -27,10 +28,7 @@ then    echo ""
 	exit 1;
 fi
 rm -f os.h
-
-# In order to accommodate for the fudge below, copy the file instead of
-# symlinking it. Otherwise we pollute the clean copy with the fudge.
-cp -p ../OS/os.h-$os os.h || exit 1
+ln -s ../OS/os.h-$os os.h || exit 1
 
 # Special-purpose fudge for older versions of Linux (pre 2.1.15) that
 # use the structure name "options" instead of "ip_options".
